@@ -69,7 +69,7 @@ simtech_act_to_mm_act (int nsmod)
 }
 
 static void
-simtech_tech_changed (MMAtSerialPort *port,
+simtech_tech_changed (MMPortSerialAt *port,
                       GMatchInfo *match_info,
                       MMBroadbandModemSimtech *self)
 {
@@ -88,7 +88,7 @@ static void
 set_unsolicited_events_handlers (MMBroadbandModemSimtech *self,
                                  gboolean enable)
 {
-    MMAtSerialPort *ports[2];
+    MMPortSerialAt *ports[2];
     guint i;
     GRegex *regex;
 
@@ -104,10 +104,10 @@ set_unsolicited_events_handlers (MMBroadbandModemSimtech *self,
             continue;
 
         /* Access technology related */
-        mm_at_serial_port_add_unsolicited_msg_handler (
+        mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
             regex,
-            enable ? (MMAtSerialUnsolicitedMsgFn)simtech_tech_changed : NULL,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)simtech_tech_changed : NULL,
             enable ? self : NULL,
             NULL);
     }
@@ -135,7 +135,7 @@ parent_setup_unsolicited_events_ready (MMIfaceModem3gpp *self,
     else {
         /* Our own setup now */
         set_unsolicited_events_handlers (MM_BROADBAND_MODEM_SIMTECH (self), TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     }
 
     g_simple_async_result_complete (simple);
@@ -171,7 +171,7 @@ parent_cleanup_unsolicited_events_ready (MMIfaceModem3gpp *self,
     if (!iface_modem_3gpp_parent->cleanup_unsolicited_events_finish (self, res, &error))
         g_simple_async_result_take_error (simple, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     g_simple_async_result_complete (simple);
     g_object_unref (simple);
 }
@@ -244,6 +244,7 @@ parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
         g_simple_async_result_take_error (simple, error);
         g_simple_async_result_complete (simple);
         g_object_unref (simple);
+        return;
     }
 
     /* Our own enable now */
